


#Clear existing data and graphics
rm(list=ls())


diagn <- function(x=1, nrow=length(x), ncol=nrow)
  if(is.matrix(x)) diag(x=x) else diag(x=x, nrow=nrow, ncol=ncol)

orm.scores <- function (y, X, link=c("logistic", "probit", "cauchit", "loglog", "cloglog")) {
  y <- as.numeric(factor(y))
  if(!is.matrix(X)) X = matrix(X, ncol=1)
  N = length(y)
  ny = length(table(y))
  na = ny - 1
  nb = ncol(X)
  npar = na + nb
  Z = outer(y, 1:ny, "<=")
  
  mod <- orm(y~X, family=link[1], x=TRUE, y=TRUE)
  
  alpha = mod$coeff[1:na]
  beta = mod$coeff[-(1:na)]
  
  dl.dtheta = matrix(0, N, npar)
  
  Gamma = matrix(0,N,na)
  
  dlow.dtheta = dhi.dtheta =dpij.dtheta= matrix(, npar, N)
  
  
  for (i in 1:N) {
    z = Z[i,]  ## z has length ny
    x = X[i,]
    
    gamma <-1- mod$trans$cumprob(alpha + sum(beta*x))
    diffgamma = diff(c(gamma,1))
    diffgamma <- ifelse(diffgamma<1e-16, 1e-16, diffgamma)
    invgamma = 1/gamma
    invgamma <- ifelse(invgamma==Inf, 1e16, invgamma)
    invgamma2 = invgamma^2
    invdiffgamma = 1/diffgamma
    invdiffgamma2 = invdiffgamma^2
    phi = log(gamma / diffgamma) ## phi has length na
    Gamma[i,] = gamma
    
    
    dg.dphi = 1 - 1/(1 + exp(phi))
    ## l is the log likelihood (6.3) in McCullagh (1980)
    dl.dphi = z[-ny] - z[-1] * dg.dphi
    t.dl.dphi = t(dl.dphi)
    
    
    dphi.dgamma = matrix(0,na,na)
    diag(dphi.dgamma) = invgamma + invdiffgamma
    if(na > 1)
      dphi.dgamma[cbind(1:(na-1), 2:na)] = -invdiffgamma[-na]
    
    
    tmp <- mod$trans$deriv(x=alpha + sum(beta*x), f=mod$trans$cumprob(alpha + sum(beta*x)))
    tmp <- ifelse(tmp<1e-16, 1e-16, tmp)
    dgamma.base <- - tmp
    #dgamma.base <- - mod$trans$deriv(x=alpha + sum(beta*x), f=mod$trans$cumprob(alpha + sum(beta*x)))
    
    dgamma.dalpha = diagn(dgamma.base)
    dgamma.dbeta = dgamma.base %o% x
    #dgamma.dtheta[i,,] = cbind(dgamma.dalpha, dgamma.dbeta)
    #### First derivatives of log-likelihood (score functions)    
    dl.dalpha = dl.dphi %*% dphi.dgamma %*% dgamma.dalpha
    dl.dbeta = dl.dphi %*% dphi.dgamma %*% dgamma.dbeta
    dl.dtheta[i,] = c(dl.dalpha, dl.dbeta)
    
    if (y[i] == 1) {
      dlow.dtheta[,i] <- 0
    } else {
      dlow.dtheta[,i] <- c(dgamma.dalpha[y[i]-1,], dgamma.dbeta[y[i]-1, ])
    }
    
    if (y[i] == ny) {
      dhi.dtheta[,i] <- 0
    } else {
      
      dhi.dtheta[,i] <- -c(dgamma.dalpha[y[i],], dgamma.dbeta[y[i],])
    }
    
  }
  
  low = cbind(0, Gamma)[cbind(1:N, y)]
  hi = cbind(1-Gamma, 0)[cbind(1:N, y)]
  presid <- low - hi
  dpresid.dtheta <- dlow.dtheta - dhi.dtheta
  
  result <- list(dl.dtheta=dl.dtheta,
                 d2l.dtheta.dtheta = -mod$info.matrix,
                 presid=presid,
                 dpresid.dtheta = dpresid.dtheta )
  
  
  
  return(result)
  
}

logit.scores = function(y, X) {
  
  y <- as.numeric(factor(y))
  if(!is.matrix(X)) X = matrix(X, ncol=1)
  
  ## N: number of subjects; ny: number of y categories
  N = length(y)
  ny = length(table(y))
  
  ## na, nb: number of parameters in alpha and beta
  na = ny - 1
  nb = ncol(X)
  npar = na + nb
  ## Z is defined as in McCullagh (1980) JRSSB 42:109-142
  Z = outer(y, 1:ny, "<=")
  
  ## Fit proportional odds model and obtain the MLEs of parameters.
  mod = lrm(y ~ X, tol=1e-50, maxit=100)
  alpha = -mod$coeff[1:na]
  beta = -mod$coeff[-(1:na)]
  
  ## Scores are stored for individuals.
  dl.dtheta = matrix(NA, N, npar)
  ## Information matrices are stored as sums over all individuals.
  d2l.dalpha.dalpha = matrix(0,na,na)
  d2l.dalpha.dbeta = matrix(0,na,nb)
  d2l.dbeta.dbeta = matrix(0,nb,nb)
  d2l.dbeta.dalpha = matrix(0,nb,na)
  ## Predicted probabilities p0 and dp0.dtheta are stored for individuals.
  p0 = matrix(,N,ny)
  dp0.dtheta = array(0,c(N,ny,npar))
  ## Cumulative probabilities
  Gamma = matrix(0,N,na)
  dgamma.dtheta = array(0,c(N,na,npar))
  
  for (i in 1:N) {
    z = Z[i,]  ## z has length ny
    x = X[i,]
    
    ## gamma and phi are defined as in McCullagh (1980)
    gamma = 1 - 1/(1 + exp(alpha + sum(beta*x))) ## gamma has length na
    diffgamma = diff(c(gamma,1))
    diffgamma <- ifelse(diffgamma<1e-16, 1e-16, diffgamma)
    
    invgamma = 1/gamma
    invgamma2 = invgamma^2
    invdiffgamma = 1/diffgamma
    invgamma <- ifelse(invgamma==Inf, 1e16, invgamma)
    invdiffgamma2 = invdiffgamma^2
    phi = log(gamma / diffgamma) ## phi has length na
    Gamma[i,] = gamma
    
    
    #### Some intermediate derivatives
    ## g(phi) = log(1+exp(phi))
    dg.dphi = 1 - 1/(1 + exp(phi))
    ## l is the log likelihood (6.3) in McCullagh (1980)
    dl.dphi = z[-ny] - z[-1] * dg.dphi
    t.dl.dphi = t(dl.dphi)
    
    ## dphi.dgamma is a na*na matrix with rows indexed by phi
    ## and columns indexed by gamma
    dphi.dgamma = matrix(0,na,na)
    diag(dphi.dgamma) = invgamma + invdiffgamma
    if(na > 1)
      dphi.dgamma[cbind(1:(na-1), 2:na)] = -invdiffgamma[-na]
    
    dgamma.base = gamma * (1-gamma)
    dgamma.base <- ifelse(dgamma.base<1e-16, 1e-16, dgamma.base)
    dgamma.dalpha = diagn(dgamma.base)
    dgamma.dbeta = dgamma.base %o% x
    dgamma.dtheta[i,,] = cbind(dgamma.dalpha, dgamma.dbeta)
    
    d2gamma.base = gamma * (1-gamma) * (1-2*gamma)
    
    ##
    d2l.dphi.dphi = diagn(-z[-1] * dg.dphi * (1-dg.dphi))
    d2l.dphi.dalpha = d2l.dphi.dphi %*% dphi.dgamma %*% dgamma.dalpha
    d2l.dphi.dbeta = d2l.dphi.dphi %*% dphi.dgamma %*% dgamma.dbeta
    
    ##
    d2phi.dgamma.dalpha = array(0,c(na,na,na))
    d2phi.dgamma.dalpha[cbind(1:na,1:na,1:na)] = (-invgamma2 + invdiffgamma2) * dgamma.base
    if(na > 1) {
      d2phi.dgamma.dalpha[cbind(1:(na-1),1:(na-1),2:na)] = -invdiffgamma2[-na] * dgamma.base[-1]
      d2phi.dgamma.dalpha[cbind(1:(na-1),2:na,1:(na-1))] = -invdiffgamma2[-na] * dgamma.base[-na]
      d2phi.dgamma.dalpha[cbind(1:(na-1),2:na,2:na)] = invdiffgamma2[-na] * dgamma.base[-1]
    }
    
    ##
    d2phi.dgamma.dbeta = array(0,c(na,na,nb))
    rowdiff = matrix(0,na,na)
    diag(rowdiff) = 1
    if(na > 1) rowdiff[cbind(1:(na-1),2:na)] = -1
    d2phi.dgamma.dbeta.comp1 = diagn(-invdiffgamma2) %*% rowdiff %*% dgamma.dbeta
    d2phi.dgamma.dbeta.comp2 = diagn(-invgamma2) %*% dgamma.dbeta - d2phi.dgamma.dbeta.comp1
    for(j in 1:na) {
      d2phi.dgamma.dbeta[j,j,] = d2phi.dgamma.dbeta.comp2[j,]
      if(j < na)
        d2phi.dgamma.dbeta[j,j+1,] = d2phi.dgamma.dbeta.comp1[j,]
    }
    
    ##
    d2gamma.dalpha.dbeta = array(0,c(na,na,nb))
    for(j in 1:na)
      d2gamma.dalpha.dbeta[j,j,] = d2gamma.base[j] %o% x
    
    ##
    d2gamma.dbeta.dbeta = d2gamma.base %o% x %o% x
    
    
    #### First derivatives of log-likelihood (score functions)
    dl.dalpha = dl.dphi %*% dphi.dgamma %*% dgamma.dalpha
    dl.dbeta = dl.dphi %*% dphi.dgamma %*% dgamma.dbeta
    dl.dtheta[i,] = c(dl.dalpha, dl.dbeta)
    
    
    #### Second derivatives of log-likelihood
    #### Since first derivative is a sum of terms each being a*b*c,
    #### second derivative is a sum of terms each being (a'*b*c+a*b'*c+a*b*c').
    
    #### d2l.dalpha.dalpha
    ## Obtain aprime.b.c
    ## Transpose first so that matrix multiplication is meaningful.
    ## Then transpose so that column is indexed by second alpha.
    aprime.b.c = t(crossprod(d2l.dphi.dalpha, dphi.dgamma %*% dgamma.dalpha))
    
    ## Obtain a.bprime.c
    ## run through the index of second alpha
    a.bprime.c = matrix(,na,na)
    for(j in 1:na)
      a.bprime.c[,j] = t.dl.dphi %*% d2phi.dgamma.dalpha[,,j] %*% dgamma.dalpha
    
    ## Obtain a.b.cprime
    ## cprime = d2gamma.dalpha.dalpha = 0 if indices of the two alphas differ.
    d2gamma.dalpha.dalpha = diagn(d2gamma.base)
    a.b.cprime = diagn(as.vector(dl.dphi %*% dphi.dgamma %*% d2gamma.dalpha.dalpha))
    
    ## summing over individuals
    d2l.dalpha.dalpha = aprime.b.c + a.bprime.c + a.b.cprime + d2l.dalpha.dalpha
    
    
    #### d2l.dalpha.dbeta
    aprime.b.c = t(crossprod(d2l.dphi.dbeta, dphi.dgamma %*% dgamma.dalpha))
    a.bprime.c = a.b.cprime = matrix(,na,nb)
    for(j in 1:nb) {
      a.bprime.c[,j] = t.dl.dphi %*% d2phi.dgamma.dbeta[,,j] %*% dgamma.dalpha
      a.b.cprime[,j] = t.dl.dphi %*% dphi.dgamma %*% d2gamma.dalpha.dbeta[,,j]
    }
    d2l.dalpha.dbeta = aprime.b.c + a.bprime.c + a.b.cprime + d2l.dalpha.dbeta
    
    
    
    aprime.b.c = t(crossprod(d2l.dphi.dbeta, dphi.dgamma %*% dgamma.dbeta))
    a.bprime.c = a.b.cprime = matrix(,nb,nb)
    for(j in 1:nb) {
      a.bprime.c[,j] = t.dl.dphi %*% d2phi.dgamma.dbeta[,,j] %*% dgamma.dbeta
      a.b.cprime[,j] = t.dl.dphi %*% dphi.dgamma %*% d2gamma.dbeta.dbeta[,,j]
    }
    d2l.dbeta.dbeta = aprime.b.c + a.bprime.c + a.b.cprime + d2l.dbeta.dbeta 
    
    
  }
  
  #### Final assembly
  d2l.dtheta.dtheta = rbind(
    cbind(d2l.dalpha.dalpha, d2l.dalpha.dbeta),
    cbind(t(d2l.dalpha.dbeta), d2l.dbeta.dbeta))
  
  
  npar.z = dim(dl.dtheta)[2]
  dlow.dtheta = dhi.dtheta = matrix(, npar.z, N)
  for(i in 1:N) {
    if (y[i] == 1) {
      dlow.dtheta[,i] <- 0
    } else {
      dlow.dtheta[,i] <- dgamma.dtheta[i,y[i]-1,]
    }
    
    if (y[i] == ny) {
      dhi.dtheta[,i] <- 0
    } else {
      dhi.dtheta[,i] <- -dgamma.dtheta[i,y[i],]
    }
  }
  
  low = cbind(0, Gamma)[cbind(1:N, y)]
  hi = cbind(1-Gamma, 0)[cbind(1:N, y)]
  presid <- low - hi
  dpresid.dtheta <- dlow.dtheta - dhi.dtheta
  
  result <- list(dl.dtheta=dl.dtheta,
                 d2l.dtheta.dtheta = -mod$info.matrix,
                 presid=presid,
                 dpresid.dtheta = dpresid.dtheta )
  
  return(result)
  
}




corTS.NA <-  function(ind.x, ind.y, 
                      presid.x, presid.y,
                      dl.dtheta.x, dl.dtheta.y,
                      d2l.dtheta.dtheta.x, d2l.dtheta.dtheta.y,
                      dpresid.dtheta.x, dpresid.dtheta.y,
                      fisher=TRUE){
  
  ind <- ind.x*ind.y
  n <- length(ind)
  npar.xz = dim(dl.dtheta.x)[2]
  npar.yz = dim(dl.dtheta.y)[2]
  Ntheta = npar.xz + npar.yz + 6
  xz.dl.dtheta <- matrix(0, n, npar.xz)
  xz.dl.dtheta[which(ind.x==1), ] <- dl.dtheta.x
  
  yz.dl.dtheta <- matrix(0, n, npar.yz)
  yz.dl.dtheta[which(ind.y==1), ] <- dl.dtheta.y
  
  xz.d2l.dtheta.dtheta <- as.matrix(d2l.dtheta.dtheta.x)
  yz.d2l.dtheta.dtheta <- as.matrix(d2l.dtheta.dtheta.y)
  
  xresid <- rep(0, n)
  yresid <- rep(0, n)
  xresid[which(ind.x==1)] <- presid.x
  xresid[which(ind==0)] <- 0
  
  yresid[which(ind.y==1)] <- presid.y
  yresid[which(ind==0)] <- 0
  
  
  dxresid.dthetax <- matrix(0, npar.xz, n)
  dyresid.dthetay <- matrix(0, npar.yz, n)
  
  dxresid.dthetax[, which(ind.x==1)] <- dpresid.dtheta.x
  dxresid.dthetax[, which(ind==0)] <- 0
  
  dyresid.dthetay[, which(ind.y==1)] <- dpresid.dtheta.y
  dyresid.dthetay[, which(ind.y==0)] <- 0
  
  TS = cor(xresid[which(ind==1)], yresid[which(ind==1)])
  
  xresid2 = xresid^2
  yresid2 = yresid^2
  
  
  xbyyresid <-  xresid * yresid
  mean.xresid <- mean(xresid[which(ind==1)])
  mean.yresid <- mean(yresid[which(ind==1)])
  mean.xbyyresid = mean(xbyyresid[which(ind==1)])
  mean.xresid2 <- mean(xresid[which(ind==1)]^2)
  mean.yresid2 <- mean(yresid[which(ind==1)]^2)
  
  
  bigphi <- cbind(xz.dl.dtheta,
                  yz.dl.dtheta,
                  (mean.xresid - xresid)*ind,
                  (mean.yresid - yresid)*ind,
                  (mean.xbyyresid - xbyyresid)*ind,
                  (mean.xresid2 - xresid2)*ind,
                  (mean.yresid2 - yresid2)*ind,
                  0)
  
  A = matrix(0,Ntheta,Ntheta)
  
  A[1:npar.xz, 1:npar.xz] <- xz.d2l.dtheta.dtheta
  A[npar.xz+(1:npar.yz), npar.xz+(1:npar.yz)] = yz.d2l.dtheta.dtheta
  A[Ntheta-6+(1:6), Ntheta-6+(1:6)] = diag(sum(ind), 6)
  
  bigpartial = rbind(c( dxresid.dthetax %*% ind, rep(0, npar.yz)),
                     c( rep(0, npar.xz), dyresid.dthetay %*% ind ),
                     c(dxresid.dthetax %*% yresid, dyresid.dthetay %*% xresid),
                     c(dxresid.dthetax %*% (2*xresid), rep(0, npar.yz)),
                     c(rep(0, npar.xz), dyresid.dthetay %*% (2*yresid)))
  
  
  A[Ntheta-6+(1:5), 1:(npar.xz+npar.yz)] = -bigpartial
  
  ## TS also equals numTS / sqrt(varprod) = numTS * revsvp
  numTS = mean.xbyyresid - mean.xresid * mean.yresid
  var.xresid = mean.xresid2 - mean.xresid^2
  var.yresid = mean.yresid2 - mean.yresid^2
  varprod = var.xresid * var.yresid
  revsvp = 1/sqrt(varprod)
  dTS.dvarprod = numTS * (-0.5) * revsvp^3
  
  smallpartial = 
    sum(ind)*c(-mean.yresid * revsvp + dTS.dvarprod * (-2*mean.xresid*var.yresid),
               -mean.xresid * revsvp + dTS.dvarprod * (-2*mean.yresid*var.xresid),
               revsvp,
               dTS.dvarprod * var.yresid,
               dTS.dvarprod * var.xresid)
  
  A[Ntheta, Ntheta-6+(1:5)] = -smallpartial
  
  
  
  SS = solve(A, t(bigphi))
  var.theta = tcrossprod (SS, SS)
  varTS = var.theta[Ntheta, Ntheta]
  
  pvalTS = 2 * pnorm( -abs(TS)/sqrt(varTS))
  
  
  
  if (fisher==TRUE){
    ####Fisher's transformation
    TS_f <- log( (1+TS)/(1-TS) )
    var.TS_f <- varTS*(2/(1-TS^2))^2
    pvalTS <- 2 * pnorm( -abs(TS_f)/sqrt(var.TS_f))
  }
  
  result <- c(TS, varTS, pvalTS)
  names(result) <- c("TS", "var.TS", "pval.TS")
  return(result)
}

#graphics.off()
#Load Hmisc library
library(Hmisc)
#Read Data
data=read.csv('../../../Documents/datasets/hiv-biomarkers/AIACK23Study_DATA_2016-01-12_1340.csv')
#Setting Labels

label(data$record_id)="Record ID"
label(data$aiac_pid)="AIAC Study ID"
label(data$age)="age at visit"
label(data$dob)="Date of birth"
label(data$screening_date)="Screening date"
label(data$visit_date)="Visit date"
label(data$sex)="Sex"
label(data$race)="Race"
label(data$bmi_screening)="Most recent BMI from medical record"
label(data$study_arm)="Study Arm"
label(data$hiv_suppr)="HIV RNA suppressed last check?"
label(data$date_vl_suppr)="Date of last suppressed VL"
label(data$cd4_art_start)="CD4 at ART start"
label(data$cd4_enroll)="CD4 at enrollment"
label(data$cd4_pct_enroll)="CD4 percent at enrollment"
label(data$cd8_enroll)="CD8 at enrollment"
label(data$cd3_enroll)="CD3 at enrollment"
label(data$cd4_cd8_ratio)="CD4:CD8 ratio"
label(data$cd4_enroll_date)="Date of enrollment CD4 measurement"
label(data$on_efvtvd_6m)="On EFV/TVD >6 months?"
label(data$on_art_2y)="On ART >2 years?"
label(data$art_start)="ART start date"
label(data$duration_art)="Duration of treatment"
label(data$smoker)="Smoker?"
label(data$hepc_ab_pos)="Positive Hep C antibody?"
label(data$fish_oil)="Taking fish oil or omega 3?"
label(data$niacin)="Taking niacin?"
label(data$testosterone)="Receiving testosterone injections?"
label(data$cigs_day)="Cigarettes per day"
label(data$nsaid_use)="Daily NSAIDS?"
label(data$asa_daily)="Daily aspirin?"
label(data$mvi_daily)="Multivitamin daily?"
label(data$herbal_daily)="Herbal medications daily?"
label(data$herbals)="List herbal medicaitons"
label(data$other_meds)="List other medications"
label(data$vr19521_aiac_participant_demographics_and_clinical_complete)="Complete?"
label(data$aiac_crc_pid)="AIAC Study ID"
label(data$aiac_crc_date)="Date/time of visit"
label(data$aiac_crc_fasting)="Fasting >8 hrs"
label(data$aiac_crc_preg)="Pregnancy test negative"
label(data$aiac_crc_chol)="Fasting total cholesterol"
label(data$aiac_crc_ldl)="Fasting LDL"
label(data$aiac_crc_hdl)="Fasting HDL"
label(data$aiac_crc_tg)="Fasting TG"
label(data$aiac_crc_hscrp)="Highly sensitive CRP"
label(data$aiac_crc_hba1c)="Hemoglobin A1c"
label(data$aiac_leptin)="Leptin (ng/ml)"
label(data$aiac_adiponectin)="Adiponectin (pg/ml)"
label(data$aiac_resistin)="Resistin (pg/ml)"
label(data$aiac_crc_glu_t0)="Baseline glucose"
label(data$aiac_insulin_t0)="Baseline insulin (uU/ml)"
label(data$ogtt_start)="OGTT start time"
label(data$aiac_crc_glu_t90)="90 minute glucose"
label(data$aiac_insulin_t90)="90 minute insulin (uU/ml)"
label(data$ogtt_90min)="OGTT 90 min sample time"
label(data$aiac_crc_glu_t120)="120 minute glucose"
label(data$ogtt_120min)="OGTT 120 min sample time"
label(data$aiac_homa_ir)="Baseline HOMA-IR"
label(data$homa2_ins_sens)="HOMA 2 insulin sensitivity (%S)"
label(data$homa2_beta_cell)="HOMA 2 beta cell function (%B)"
label(data$homa2_ir)="HOMA 2 insulin resistance"
label(data$crc_ogis120)="OGIS 120 (ml/min/m^2)"
label(data$r_arm_sbp)="Right arm SBP"
label(data$r_arm_dbp)="Right arm DBP"
label(data$l_arm_sbp)="Left arm SBP"
label(data$l_arm_dbp)="Left arm DBP"
label(data$aiac_crc_wt)="Weight (kg)"
label(data$aiac_crc_ht)="Height (m)"
label(data$calc_bmi)="Calculated BMI"
label(data$crc_bsa)="Body surface area (m^2)"
label(data$aiac_crc_hip_circ)="Averaged hip circumference (cm)"
label(data$aiac_crc_waist_circ)="Averaged waist circumference (cm)"
label(data$aiac_crc_muac)="Averaged mid-upper arm circ.(cm)"
label(data$aiac_crc_visit_complete)="Complete?"
label(data$aiac_crc_us_pid)="AIAC Study ID"
label(data$crc_us_date)="Date of CRC visit"
label(data$r_bulb_imt_0cm)="Right carotid bulb lateral IMT (0cm)"
label(data$r_cca_imt_1cm)="Right CCA lateral IMT (1cm)"
label(data$r_cca_imt_2cm)="Right CCA lateral IMT (2cm)"
label(data$l_bulb_imt_0cm)="Left carotid bulb lateral IMT (0cm)"
label(data$l_cca_imt_1cm)="Left CCA lateral IMT (1cm)"
label(data$l_cca_imt_2cm)="Left CCA lateral IMT (2cm)"
label(data$r_bulb_imt_zoom_0cm)="Right carotid bulb lateral IMT zoom (0cm)"
label(data$r_cca_imt_zoom_1cm)="Right CCA lateral IMT zoom (1cm)"
label(data$r_cca_imt_zoom_2cm)="Right CCA lateral IMT zoom (2cm)"
label(data$l_bulb_imt_zoom_0cm)="Left carotid bulb lateral IMT zoom (0cm)"
label(data$l_cca_imt_zoom_1cm)="Left CCA lateral IMT zoom (1cm)"
label(data$l_cca_imt_zoom_2cm)="Left CCA lateral IMT zoom (2cm)"
label(data$r_bulb_imt_qlab_0mm)="Right carotid bulb lateral IMT qlab (0mm)"
label(data$r_cca_imt_qlab_1mm)="Right CCA lateral IMT qlab (1mm)"
label(data$r_cca_imt_qlab_2mm)="Right CCA lateral IMT qlab (2mm)"
label(data$l_bulb_imt_qlab_0mm)="Left carotid bulb lateral IMT qlab (0mm)"
label(data$l_cca_imt_qlab_1mm)="Left CCA lateral IMT qlab (1mm)"
label(data$l_cca_imt_qlab_2mm)="Left CCA lateral IMT qlab (2mm)"
label(data$r_ica_imt_cm)="Right ICA lateral IMT (cm)"
label(data$r_ica_imt_zoom_cm)="Right ICA lateral IMT zoom (cm)"
label(data$r_ica_imt_qlab_mm)="Right ICA lateral IMT qlab (mm)"
label(data$l_ica_imt_cm)="Left ICA lateral IMT (cm)"
label(data$l_ica_imt_zoom_cm)="Left ICA lateral IMT zoom (cm)"
label(data$l_ica_imt_qlab_mm)="Left ICA lateral IMT qlab (mm)"
label(data$r_cca_plaques)="Right CCA plaques"
label(data$r_bulb_plaques)="Right bulb plaques"
label(data$r_ica_plaques)="Right ICA plaques"
label(data$l_cca_plaques)="Left CCA plaques"
label(data$l_bulb_plaques)="Left bulb plaques"
label(data$l_ica_plaques)="Left ICA plaques"
label(data$bart_dia_prehyperem)="Brachial artery diameter pre-hyperemia (cm)"
label(data$bart_psflow_prehyperem)="Brachial artery peak systolic flow pre-hyperemia (cm/s)"
label(data$bart_pdflow_prehyperem)="Brachial artery peak diastolic flow pre-hyperemia (cm/s)"
label(data$bart_psflow_posthyperem)="Brachial artery peak systolic flow post-hyperemia (cm/s)"
label(data$bart_pdflow_posthyperem)="Brachial artery peak diastolic flow post-hyperemia (cm/s)"
label(data$bart_dia_30sec)="Brachial artery diameter post-hyperemia - 30 sec (cm)"
label(data$bart_dia_60sec)="Brachial artery diameter post-hyperemia - 60 sec (cm)"
label(data$bart_dia_90sec)="Brachial artery diameter post-hyperemia - 90 sec (cm)"
label(data$aiac_crc_visit_ultrasound_measurements_complete)="Complete?"
label(data$aiac_id_dexa)="AIAC ID"
label(data$left_arm_bone_mass)="Left Arm Bone Mass"
label(data$left_leg_bone_mass)="Left Leg Bone Mass"
label(data$left_trunk_bone_mass)="Left Trunk Bone Mass"
label(data$left_total_bone_mass)="Left Total Bone Mass"
label(data$rt_arm_bone_mass)="Right Arm Bone Mass"
label(data$rt_leg_bone_mass)="Right Leg Bone Mass"
label(data$rt_trunk_bone_mass)="Right Trunk Bone Mass"
label(data$rt_total_bone_mass)="Right Total Bone Mass"
label(data$arms_bone_mass)="Arms Bone Mass"
label(data$arms_bone_mass_diff)="Arms Bone Mass Difference"
label(data$legs_bone_mass)="Legs Bone Mass"
label(data$legs_bone_mass_diff)="Legs Bone Mass Difference"
label(data$trunk_bone_mass)="Trunk Bone Mass"
label(data$trunk_bone_mass_diff)="Trunk Bone Mass Difference"
label(data$android_bone_mass)="Android Bone Mass"
label(data$gynoid_bone_mass)="Gynoid Bone Mass"
label(data$total_bone_mass)="Total Bone Mass"
label(data$total_bone_mass_diff)="Total Bone Mass Difference"
label(data$left_arm_fat_mass)="Left Arm Fat Mass"
label(data$left_leg_fat_mass)="Left Leg Fat Mass"
label(data$left_trunk_fat_mass)="Left Trunk Fat Mass"
label(data$left_total_fat_mass)="Left Total Fat Mass"
label(data$rt_arm_fat_mass)="Right Arm Fat Mass"
label(data$rt_leg_fat_mass)="Right Leg Fat Mass"
label(data$rt_trunk_fat_mass)="Right Trunk Fat Mass"
label(data$rt_total_fat_mass)="Right Total Fat Mass"
label(data$arms_fat_mass)="Arms Fat Mass"
label(data$arms_fat_mass_diff)="Arms Fat Mass Difference"
label(data$legs_fat_mass)="Legs Fat Mass"
label(data$legs_fat_mass_diff)="Legs Fat Mass Difference"
label(data$trunk_fat_mass)="Trunk Fat Mass"
label(data$trunk_fat_mass_diff)="Trunk Fat Mass Difference"
label(data$android_fat_mass)="Android Fat Mass"
label(data$gynoid_fat_mass)="Gynoid Fat Mass"
label(data$total_fat_mass)="Total Fat Mass"
label(data$total_fat_mass_diff)="Total Fat Mass Difference"
label(data$left_arm_lean_mass)="Left Arm Lean Mass"
label(data$left_leg_lean_mass)="Left Leg Lean Mass"
label(data$left_trunk_lean_mass)="Left Trunk Lean Mass"
label(data$left_total_lean_mass)="Left Total Lean Mass"
label(data$rt_arm_lean_mass)="Right Arm Lean Mass"
label(data$rt_leg_lean_mass)="Right Leg Lean Mass"
label(data$rt_trunk_lean_mass)="Right Trunk Lean Mass"
label(data$rt_total_lean_mass)="Right Total Lean Mass"
label(data$arms_lean_mass)="Arms Lean Mass"
label(data$arms_lean_mass_diff)="Arms Lean Mass Difference"
label(data$legs_lean_mass)="Legs Lean Mass"
label(data$legs_lean_mass_diff)="Legs Lean Mass Difference"
label(data$trunk_lean_mass)="Trunk Lean Mass"
label(data$trunk_lean_mass_diff)="Trunk Lean Mass Difference"
label(data$android_lean_mass)="Android Lean Mass"
label(data$gynoid_lean_mass)="Gynoid Lean Mass"
label(data$total_lean_mass)="Total Lean Mass"
label(data$total_lean_mass_diff)="Total Lean Mass Difference"
label(data$vat_cm)="Visceral Adipose Tissue (cm)"
label(data$vat_g)="Visceral Adipose Tissue (grams)"
label(data$aiac_crc_dexa_measurements_complete)="Complete?"
label(data$aiac_id_immuno)="AIAC ID"
label(data$scd14)="Soluble CD14 (pg/ml)"
label(data$scd163)="Soluble CD163 (ng/ml)"
label(data$inf_gamma)="Interferon-gamma (pg/ml)"
label(data$il_10)="IL-10 (pg/ml)"
label(data$il_12p70)="IL-12p70 (pg/ml)"
label(data$il_1_beta)="IL-1 beta (pg/ml)"
label(data$il_6)="IL-6 (pg/ml)"
label(data$il_8)="IL-8 (pg/ml)"
label(data$tnf_alpha)="TNF-alpha (pg/ml)"
label(data$il_2)="IL-2 (pg/ml)"
label(data$il_4)="IL-4 (pg/ml)"
label(data$il_13)="IL-13 (pg/ml)"
label(data$crp_pgml)="C-reactive protein (pg/ml)"
label(data$saa_pgml)="Serum amyloid A (pg/ml)"
label(data$icam1)="Intercellular adhesion molecule-1 (pg/ml)"
label(data$vcam1)="Vascular cell adhesion molecule-1 (pg/ml)"
label(data$il_17)="IL-17 (pg/ml)"
label(data$il_5)="IL-5 (pg/ml)"
label(data$mcp1)="Macrophage chemotactic protein-1 (pg/ml)"
label(data$mip_1a)="Macrophage inflammatory protein-1 alpha (pg/ml)"
label(data$mip_1b)="Macrophage inflammatory protein-1 beta (pg/ml)"
label(data$tnf_a_r1)="TNF-alpha receptor 1 (pg/ml)"
label(data$tnf_a_r2)="TNF-alpha receptor 2 (pg/ml)"
label(data$aiac_immunology_core_data_complete)="Complete?"
label(data$metabo_aa_aiac_id)="AIAC ID"
label(data$methylhistidine)="1-Methylhistidine (uM)"
label(data$methylhist)="3-Methylhistidine (uM)"
label(data$alanine)="Alanine (uM)"
label(data$arginine)="Arginine (uM)"
label(data$asparagine)="Asparagine (uM)"
label(data$aspartic_acid)="Aspartic acid (uM)"
label(data$citrulline)="Citrulline (uM)"
label(data$glutamic_acid)="Glutamic acid (uM)"
label(data$glutamine)="Glutamine (uM)"
label(data$glycine)="Glycine (uM)"
label(data$histidine)="Histidine (uM)"
label(data$isoleucine)="Isoleucine (uM)"
label(data$leucine)="Leucine (uM)"
label(data$lysine)="Lysine (uM)"
label(data$ornithine)="Ornithine (uM)"
label(data$phenylalanine)="Phenylalanine (uM)"
label(data$proline)="Proline (uM)"
label(data$serine)="Serine (uM)"
label(data$threonine)="Threonine (uM)"
label(data$tryptophan)="Tryptophan (uM)"
label(data$tyrosine)="Tyrosine (uM)"
label(data$valine)="Valine (uM)"
label(data$aiac_metabolomics_amino_acid_data_complete)="Complete?"
label(data$metab_acyl_aiac_id)="AIAC ID"
label(data$ac_c2)="C2 (uM)"
label(data$ac_c3)="C3  (uM)"
label(data$ac_c4_butyryl)="C4 Butyryl (uM)"
label(data$ac_c4_isobutyryl)="C4 Isobutyryl (uM)"
label(data$ac_c4_dc_succinyl)="C4-DC Succinyl (uM)"
label(data$ac_c4_oh_butyryl)="C4-OH Butyryl  (uM)"
label(data$ac_c5_2_methylbutyryl)="C5 2-Methylbutyryl (uM)"
label(data$ac_c5_isovaleryl)="C5 Isovaleryl (uM)"
label(data$ac_c5_valeryl)="C5 Valeryl (uM)"
label(data$ac_c5_1)="C5:1 (uM)"
label(data$ac_c5_oh)="C5-OH (uM)"
label(data$ac_c6)="C6 (uM)"
label(data$ac_c8)="C8 (uM)"
label(data$ac_c8_oh)="C8-OH (uM)"
label(data$ac_c10)="C10 (uM)"
label(data$ac_c10_oh)="C10-OH (uM)"
label(data$ac_c12)="C12 (uM)"
label(data$ac_c14_1)="C14:1 (uM)"
label(data$ac_c14_2)="C14:2 (uM)"
label(data$ac_c14)="C14 (uM)"
label(data$ac_c16_2)="C16:2 (uM)"
label(data$ac_c16)="C16 (uM)"
label(data$ac_c18_1)="C18:1 (uM)"
label(data$ac_c18_2)="C18:2 (uM)"
label(data$ac_c18)="C18 (uM)"
label(data$aiac_metabolomics_acylcarnitine_data_complete)="Complete?"
label(data$metab_oa_aiac_id)="AIAC ID"
label(data$oa_2_hba)="2-Hydroxybutyric acid uM"
label(data$oa_lactate)="Lactate uM"
label(data$oa_pyruvate)="Pyruvate uM"
label(data$oa_3_hba)="3-Hydroxybutyric acid (uM)"
label(data$oa_malate)="Malate uM"
label(data$oa_akg)="Alpha-ketoglutarate uM"
label(data$oa_citrate)="Citrate uM"
label(data$aiac_metabolomics_organic_acid_data_complete)="Complete?"
label(data$aiac_ffa)="AIAC Free Fatty Acid ID"
label(data$ffa_hemolysis)="Sample hemolyzed?"
label(data$ffa_14_0)="14:0 (Myristic Acid) ug/ml "
label(data$ffa_16_0)="16:0 (Palmitic Acid) ug/ml"
label(data$ffa_16_1)="16:1 (Palmitoleic acid) ug/ml"
label(data$ffa_18_0)="18:0 (Stearic Acid) ug/ml"
label(data$ffa_18_1_w9)="18:1w9 (Oleic Acid) ug/ml"
label(data$ffa_18_1_w7)="18:1w7 (Vaccenic Acid) ug/ml"
label(data$ffa_18_2)="18:2 (Linoleic acid) ug/ml"
label(data$ffa_18_3_w3)="18:3w3 (a-Linolenic Acid) ug/ml"
label(data$ffa_20_4)="20:4 (Arachidonic Acid) ug/ml"
label(data$ffa_20_5)="20:5 (Eicosapentaenoic Acid) ug/ml"
label(data$total_ffa)="Total FFA ug/ml"
label(data$aiac_free_fatty_acid_data_complete)="Complete?"
label(data$aiac_flow_id)="AIAC ID"
label(data$cd4_flow_date)="CD4 Flow Cytometry Date"
label(data$cd4_percent)="CD4+ %"
label(data$cd4_pd1_percent)="CD4+ PD-1+ %"
label(data$cd4_cd38_percent)="CD4+ CD38+ %"
label(data$cd4_dr_percent)="CD4+ DR+ %"
label(data$cd4_cd38_dr_percent)="CD4+ CD38+ DR+ %"
label(data$cd4_cd57_percent)="CD4+ CD57+ %"
label(data$cd4_cd57_pd1_percent)="CD4+ CD57+ PD-1+ %"
label(data$cd4_cd69_percent)="CD4+ CD69+ %"
label(data$cd8_flow_date)="CD8+ Flow Cytometry Date"
label(data$cd8_percent)="CD8+ %"
label(data$cd8_pd1_percent)="CD8+ PD-1+ %"
label(data$cd8_cd38_percent)="CD8+ CD38+ %"
label(data$cd8_dr_percent)="CD8+ DR+ %"
label(data$cd8_cd38_dr_percent)="CD8+ CD38+ DR+ %"
label(data$cd8_cd69_percent)="CD8+ CD69+ %"
label(data$cd8_cd57_percent)="CD8+ CD57+ %"
label(data$cd8_cd57_pd1_percent)="CD8+ CD57+ PD-1+ %"
label(data$aiac_cd4cd8_flow_cytometry_data_complete)="Complete?"
label(data$aiac_mem_id)="AIAC ID"
label(data$cd3_pct_mem_panel)="CD3+ %"
label(data$cd4_pct_mem_panel)="CD4+ %"
label(data$cd4_ro)="CD4+ RO+ %"
label(data$cd4_tcm)="CD4+ TCM %"
label(data$cd4_ttm)="CD4+ TTM %"
label(data$cd4_tem)="CD4+ TEM %"
label(data$cd4_ro_pd1_mfi)="CD4+ RO+ PD1+ MFI"
label(data$cd4_ro_neg)="CD4+ RO- %"
label(data$cd4_naive)="CD4+ naive %"
label(data$cd4_intermed)="CD4+ intermediate %"
label(data$cd4_effector)="CD4+ effector %"
label(data$cd4_pd1_mfi)="CD4+ PD1+ MFI"
label(data$cd8_pct_mem_panel)="CD8+ %"
label(data$cd8_ro)="CD8+ RO+ %"
label(data$cd8_tcm)="CD8+ TCM %"
label(data$cd8_ttm)="CD8+ TTM %"
label(data$cd8_tem)="CD8+ TEM %"
label(data$cd8_ro_pd1_mfi)="CD8+ RO+ PD1+ MFI"
label(data$cd8_ro_neg)="CD8+ RO- %"
label(data$cd8_naive)="CD8+ naïve %"
label(data$cd8_intermed)="CD8+ intermediate %"
label(data$cd8_effector)="CD8+ effector %"
label(data$cd8_pd1_mfi)="CD8+ PD1+ MFI"
label(data$aiac_cd4cd8_memory_complete)="Complete?"
label(data$aiac_id_treg)="AIAC ID Treg"
label(data$cd4_pct)="CD4+ %"
label(data$cd4_treg_pct)="CD4% T reg %"
label(data$cd4_treg_cd39)="CD4+Treg+CD39+ %"
label(data$cd4_treg_pd1_pct)="CD4+Treg+PD-1+ %"
label(data$cd4_treg_cd39_pd_1)="CD4+Treg+CD39+PD-1+ %"
label(data$cd4_non_treg)="CD4+ non-T reg %"
label(data$cd4_non_treg_cd39)="CD4+ non-T reg CD39+ %"
label(data$cd4_non_treg_pd1)="CD4+ non-T reg PD-1+ %"
label(data$cd4_non_treg_cd39_pd1)="CD4+ non-T reg CD39+ PD-1+ %"
label(data$cd4_pd1)="CD4+PD-1+ %"
label(data$aiac_cd4_t_regulatory_complete)="Complete?"
#Setting Units


#Setting Factors(will create new variable for factors)
data$sex.factor = factor(data$sex,levels=c("0","1"))
data$race.factor = factor(data$race,levels=c("0","1","2","3","4"))
data$study_arm.factor = factor(data$study_arm,levels=c("1","2","3"))
data$hiv_suppr.factor = factor(data$hiv_suppr,levels=c("1","0"))
data$on_efvtvd_6m.factor = factor(data$on_efvtvd_6m,levels=c("1","0"))
data$on_art_2y.factor = factor(data$on_art_2y,levels=c("1","0"))
data$smoker.factor = factor(data$smoker,levels=c("1","0"))
data$hepc_ab_pos.factor = factor(data$hepc_ab_pos,levels=c("1","0"))
data$fish_oil.factor = factor(data$fish_oil,levels=c("1","0"))
data$niacin.factor = factor(data$niacin,levels=c("1","0"))
data$testosterone.factor = factor(data$testosterone,levels=c("1","0"))
data$nsaid_use.factor = factor(data$nsaid_use,levels=c("1","0"))
data$asa_daily.factor = factor(data$asa_daily,levels=c("1","0"))
data$mvi_daily.factor = factor(data$mvi_daily,levels=c("1","0"))
data$herbal_daily.factor = factor(data$herbal_daily,levels=c("1","0"))
data$vr19521_aiac_participant_demographics_and_clinical_complete.factor = factor(data$vr19521_aiac_participant_demographics_and_clinical_complete,levels=c("0","1","2"))
data$aiac_crc_fasting.factor = factor(data$aiac_crc_fasting,levels=c("1","0"))
data$aiac_crc_preg.factor = factor(data$aiac_crc_preg,levels=c("1","2","3"))
data$aiac_crc_visit_complete.factor = factor(data$aiac_crc_visit_complete,levels=c("0","1","2"))
data$aiac_crc_visit_ultrasound_measurements_complete.factor = factor(data$aiac_crc_visit_ultrasound_measurements_complete,levels=c("0","1","2"))
data$aiac_crc_dexa_measurements_complete.factor = factor(data$aiac_crc_dexa_measurements_complete,levels=c("0","1","2"))
data$aiac_immunology_core_data_complete.factor = factor(data$aiac_immunology_core_data_complete,levels=c("0","1","2"))
data$aiac_metabolomics_amino_acid_data_complete.factor = factor(data$aiac_metabolomics_amino_acid_data_complete,levels=c("0","1","2"))
data$aiac_metabolomics_acylcarnitine_data_complete.factor = factor(data$aiac_metabolomics_acylcarnitine_data_complete,levels=c("0","1","2"))
data$aiac_metabolomics_organic_acid_data_complete.factor = factor(data$aiac_metabolomics_organic_acid_data_complete,levels=c("0","1","2"))
data$aiac_free_fatty_acid_data_complete.factor = factor(data$aiac_free_fatty_acid_data_complete,levels=c("0","1","2"))
data$aiac_cd4cd8_flow_cytometry_data_complete.factor = factor(data$aiac_cd4cd8_flow_cytometry_data_complete,levels=c("0","1","2"))
data$aiac_cd4cd8_memory_complete.factor = factor(data$aiac_cd4cd8_memory_complete,levels=c("0","1","2"))
data$aiac_cd4_t_regulatory_complete.factor = factor(data$aiac_cd4_t_regulatory_complete,levels=c("0","1","2"))

levels(data$sex.factor)=c("Male","Female")
levels(data$race.factor)=c("caucasian","african american","hispanic","asian","other")
levels(data$study_arm.factor)=c("HIV-infected non-obese","HIV-infected obese","Uninfected obese")
levels(data$hiv_suppr.factor)=c("Yes","No")
levels(data$on_efvtvd_6m.factor)=c("Yes","No")
levels(data$on_art_2y.factor)=c("Yes","No")
levels(data$smoker.factor)=c("Yes","No")
levels(data$hepc_ab_pos.factor)=c("Yes","No")
levels(data$fish_oil.factor)=c("Yes","No")
levels(data$niacin.factor)=c("Yes","No")
levels(data$testosterone.factor)=c("Yes","No")
levels(data$nsaid_use.factor)=c("Yes","No")
levels(data$asa_daily.factor)=c("Yes","No")
levels(data$mvi_daily.factor)=c("Yes","No")
levels(data$herbal_daily.factor)=c("Yes","No")
levels(data$vr19521_aiac_participant_demographics_and_clinical_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_crc_fasting.factor)=c("Yes","No")
levels(data$aiac_crc_preg.factor)=c("No","Yes","Not applicable")
levels(data$aiac_crc_visit_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_crc_visit_ultrasound_measurements_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_crc_dexa_measurements_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_immunology_core_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_metabolomics_amino_acid_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_metabolomics_acylcarnitine_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_metabolomics_organic_acid_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_free_fatty_acid_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_cd4cd8_flow_cytometry_data_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_cd4cd8_memory_complete.factor)=c("Incomplete","Unverified","Complete")
levels(data$aiac_cd4_t_regulatory_complete.factor)=c("Incomplete","Unverified","Complete")


library(shiny)
library(ggplot2)
library(reshape2)
library(rms)
library(stats)

d <- data
d$bmi<-data$bmi_screening
d$cd4<-data$cd4_enroll
d$art.duration<-data$duration_art
study.arm<-data$study_arm.factor
d$hiv<-ifelse(study.arm=="Uninfected obese",0,1)
d$log.art.duration<-with(d, log(art.duration))

d$race<-with(d,ifelse(race>1,1,race))

d2 <- subset(d, hiv==1)

adj.var <- d2[, c("age", "sex", "race", "bmi","smoker", "cd4", "log.art.duration")]

biomarker <- d2[, c("aiac_crc_hba1c", "homa2_ins_sens",
                    "homa2_beta_cell", "crc_ogis120", "glutamic_acid", "glutamine",
                    "leucine","isoleucine", "phenylalanine", "tryptophan", "tyrosine", "valine", "ac_c3",
                    "ac_c5_2_methylbutyryl","ac_c5_isovaleryl", "ac_c5_valeryl", "oa_2_hba",
                    "oa_lactate", "oa_3_hba", "oa_akg", "oa_pyruvate")]




d2 <- biomarker


spearman.est <- spearman.pval <-  matrix(NA, nrow=dim(d2)[2], ncol=dim(d2)[2])

for(i in 1: (dim(d2)[2]-1)){
  for (j in (i+1):dim(d2)[2]){
    d.tmp <- cbind(d2[, c(i, j)])   
    d.tmp <- d.tmp[complete.cases(d.tmp), ]
    result <- cor.test(d.tmp[,1], d.tmp[,2], method="spearman", exact=FALSE)
    spearman.est[j, i] <- result$estimate
    spearman.pval[j, i] <- result$p.value    
  }
}



index <- matrix(NA, dim(d2)[1], dim(d2)[2])

for(i in 1:dim(index)[2]){
  index[,i] <- !is.na(d2[,i])
}

Sys.time()
presid <- dl.dtheta <- d2l.dtheta.dtheta <- dpresid.dtheta <- list()
for (i in 1:dim(d2)[2]){
  tryCatch({
    
    d.tmp <- data.frame(d2[,i], adj.var)
    colnames(d.tmp)[1] <- colnames(d2)[i]
    d.tmp.c <- d.tmp[complete.cases(d.tmp), ]
    X=model.matrix(~ age+sex+race+bmi+cd4+log.art.duration+smoker, d.tmp.c)
    #mod <- orm(d2[index[,i],i]~ urban + age + marital + religion + language, data=d.tmp.c )
    #X=model.matrix(~ urban, d.tmp.c)
    zzint <- match("(Intercept)", colnames(X), nomatch = 0L)
    if(zzint > 0L) {
      X <- X[, -zzint, drop = FALSE]
    }
    
    if(length(unique(d2[index[,i],i]))>2 ){
      score <- orm.scores(d.tmp.c[,1], X)
      dl.dtheta[[i]] <- score$dl.dtheta
      d2l.dtheta.dtheta[[i]] <- score$d2l.dtheta.dtheta
      presid[[i]] <- score$presid
      dpresid.dtheta[[i]] <- score$dpresid.dtheta 
    } else if(length(unique(d2[index[,i],i]))==2){
      score <- logit.scores(d.tmp.c[,1], X)
      dl.dtheta[[i]] <- score$dl.dtheta
      d2l.dtheta.dtheta[[i]] <- score$d2l.dtheta.dtheta
      presid[[i]] <- score$presid
      dpresid.dtheta[[i]] <- score$dpresid.dtheta 
    }
    
  }, error=function(e) print(paste(i, colnames(d2)[i],":", e)) )
  
  
}
Sys.time()


ts.est <- ts.pval <- matrix(NA, nrow=dim(d2)[2], ncol=dim(d2)[2])

for(i in 1: (dim(d2)[2]-1)){
  for (j in (i+1):dim(d2)[2]){
    tryCatch({
      result <- corTS.NA(ind.x=index[,i],
                         ind.y=index[,j], 
                         presid.x=presid[[i]],
                         presid.y=presid[[j]],
                         dl.dtheta.x=dl.dtheta[[i]],
                         dl.dtheta.y=dl.dtheta[[j]],
                         d2l.dtheta.dtheta.x=d2l.dtheta.dtheta[[i]],
                         d2l.dtheta.dtheta.y=d2l.dtheta.dtheta[[j]],
                         dpresid.dtheta.x=dpresid.dtheta[[i]], 
                         dpresid.dtheta.y=dpresid.dtheta[[j]],
                         fisher=TRUE)
      ts.est[i, j] <- result[1]
      #ts.var[i, j] <- result[2]
      ts.pval[i, j] <- result[3]

      
      
    }, error=function(e) print(paste(i, "-", j, ":", e)) )
    
  }
  print(paste(i, " :",Sys.time()))
}

diag(ts.est) <- rep(1, dim(ts.est)[2])
diag(ts.pval) <- rep(1, dim(ts.pval)[2])



ts.est[lower.tri(ts.est)] <- spearman.est[lower.tri(spearman.est)]
ts.pval[lower.tri(ts.pval)] <- spearman.pval[lower.tri(spearman.pval)]

####upper.tri: spearman; lower.tri: corPSR
ts.est <- t(ts.est)
ts.pval <- t(ts.pval)

x.index <- y.index <- 1:21
dat2 <- melt(ts.est[x.index, y.index])

dat.pval <- melt(ts.pval[x.index, y.index])

tmp <- subset(dat.pval, value<0.05)
tmp$x <- tmp$Var1-0.5
tmp$y <- tmp$Var2-0.5
tmp$id <- 1:dim(tmp)[1]

tmp2 <- tmp
tmp2$y <- tmp$Var2+0.5

tmp3 <- tmp
tmp3$x <- tmp$Var1+0.5
tmp3$y <- tmp$Var2+0.5

tmp4 <- tmp
tmp4$x <- tmp$Var1+0.5

pval.plot <- rbind(tmp, tmp2, tmp3, tmp4)
pval.plot <- pval.plot[order(pval.plot$id),c("id", "x", "y")]
label(d2$aiac_crc_hba1c)<- "Hemoglobin A1c (HbA1c)"
label(d2$homa2_ins_sens)<- "HOMA 2 insulin sensitivity (H2IS)"
label(d2$homa2_beta_cell)<- "HOMA 2 beta cell function (H2BF)" 
label(d2$crc_ogis120)<- "OGIS 120 (OGIS)" 
label(d2$glutamic_acid)<- "Glutamic acid (Glu)"
label(d2$glutamine)<- "Glutamine (Gln)"
label(d2$leucine)<- "Leucine (Leu)"
label(d2$isoleucine)<- "Isoleucine (Ile)"
label(d2$phenylalanine)<- "Phenylalanine (Phe)" 
label(d2$tryptophan)<- "Tryptophan (Trp)" 
label(d2$tyrosine)<- "Tyrosine (Tyr)" 
label(d2$valine)<- "Valine (Val)"
label(d2$ac_c3)<- "C3"
label(d2$ac_c5_2_methylbutyryl)<- "C5 2-Methylbutyryl (C5M)"
label(d2$ac_c5_isovaleryl)<- "C5 Isovaleryl (C5I)"
label(d2$ac_c5_valeryl)<- "C5 Valeryl (C5V)"
label(d2$oa_2_hba ) <- "2-Hydroxybutyric acid (2HbA)"
label(d2$oa_lactate) <- "Lactate (Lac)"
label(d2$oa_3_hba) <- "3-Hydroxybutyric acid (3HbA)"
label(d2$oa_akg) <- "Alpha-ketoglutarate (AKG)"
label(d2$oa_pyruvate) <- "Pyruvate (Pyr)"




postscript("biomarker_heatmap_logit.eps", horizontal = FALSE, onefile = FALSE, paper = "special",
           height = 8, width = 11)

plot1 <- ggplot(dat2, aes(Var1, Var2, fill = value)) + geom_tile() + xlab("") + ylab("")+
  scale_fill_gradient2(low = "blue",  high = "red",limits=c(-1, 1))
  
plot1 <- plot1+geom_text(aes(fill = dat2$value, label = round(dat2$value, 2)), size=3)
  

plot1  <- plot1 + theme(plot.margin=unit(c(0,0,0,0),"mm"), axis.ticks = element_blank()) 

plot1 <- plot1+ geom_polygon(aes(x=x, y=y, group=id),fill=NA, data=pval.plot, inherit.aes=FALSE, color = "black")



  
plot1 <- plot1+
  scale_x_continuous(breaks=cumsum(rep(1, dim(d2)[2])), labels=c("HbA1c", "H2IS", "H2BF", "OGIS", "Glu", "Gln", "Leu", "Ile", "Phe", "Trp", "Tyr", "Val", "C3", "C5M", "C5I", "C5V", "2HA", "Lac", "3HbA", "AKG", "Pyr")) +theme(axis.text.x = element_text(angle = 90, hjust = 1))+
  scale_y_continuous(breaks=cumsum(rep(1, dim(d2)[2])), labels=label(d2))
  
plot1
dev.off()

###### repeat the analysis with cloglog link
###### using the cloglog link function
Sys.time()
presid <- dl.dtheta <- d2l.dtheta.dtheta <- dpresid.dtheta <- list()
for (i in 1:dim(d2)[2]){
  tryCatch({
    
    d.tmp <- data.frame(d2[,i], adj.var)
    colnames(d.tmp)[1] <- colnames(d2)[i]
    d.tmp.c <- d.tmp[complete.cases(d.tmp), ]
    X=model.matrix(~ age+sex+race+bmi+cd4+log.art.duration+smoker, d.tmp.c)
    #mod <- orm(d2[index[,i],i]~ urban + age + marital + religion + language, data=d.tmp.c )
    #X=model.matrix(~ urban, d.tmp.c)
    zzint <- match("(Intercept)", colnames(X), nomatch = 0L)
    if(zzint > 0L) {
      X <- X[, -zzint, drop = FALSE]
    }
    
    if(length(unique(d2[index[,i],i]))>2 ){
      score <- orm.scores(d.tmp.c[,1], X,link="cloglog")
      dl.dtheta[[i]] <- score$dl.dtheta
      d2l.dtheta.dtheta[[i]] <- score$d2l.dtheta.dtheta
      presid[[i]] <- score$presid
      dpresid.dtheta[[i]] <- score$dpresid.dtheta 
    } else if(length(unique(d2[index[,i],i]))==2){
      score <- logit.scores(d.tmp.c[,1], X)
      dl.dtheta[[i]] <- score$dl.dtheta
      d2l.dtheta.dtheta[[i]] <- score$d2l.dtheta.dtheta
      presid[[i]] <- score$presid
      dpresid.dtheta[[i]] <- score$dpresid.dtheta 
    }
    
  }, error=function(e) print(paste(i, colnames(d2)[i],":", e)) )
  
  
}
Sys.time()


ts.est <- ts.pval <- matrix(NA, nrow=dim(d2)[2], ncol=dim(d2)[2])

for(i in 1: (dim(d2)[2]-1)){
  for (j in (i+1):dim(d2)[2]){
    tryCatch({
      result <- corTS.NA(ind.x=index[,i],
                         ind.y=index[,j], 
                         presid.x=presid[[i]],
                         presid.y=presid[[j]],
                         dl.dtheta.x=dl.dtheta[[i]],
                         dl.dtheta.y=dl.dtheta[[j]],
                         d2l.dtheta.dtheta.x=d2l.dtheta.dtheta[[i]],
                         d2l.dtheta.dtheta.y=d2l.dtheta.dtheta[[j]],
                         dpresid.dtheta.x=dpresid.dtheta[[i]], 
                         dpresid.dtheta.y=dpresid.dtheta[[j]],
                         fisher=TRUE)
      ts.est[i, j] <- result[1]
      #ts.var[i, j] <- result[2]
      ts.pval[i, j] <- result[3]
      
      
      
    }, error=function(e) print(paste(i, "-", j, ":", e)) )
    
  }
  print(paste(i, " :",Sys.time()))
}

diag(ts.est) <- rep(1, dim(ts.est)[2])
diag(ts.pval) <- rep(1, dim(ts.pval)[2])



ts.est[lower.tri(ts.est)] <- spearman.est[lower.tri(spearman.est)]
ts.pval[lower.tri(ts.pval)] <- spearman.pval[lower.tri(spearman.pval)]

####upper.tri: spearman; lower.tri: corPSR
ts.est <- t(ts.est)
ts.pval <- t(ts.pval)

x.index <- y.index <- 1:21
dat2 <- melt(ts.est[x.index, y.index])

dat.pval <- melt(ts.pval[x.index, y.index])

tmp <- subset(dat.pval, value<0.05)
tmp$x <- tmp$Var1-0.5
tmp$y <- tmp$Var2-0.5
tmp$id <- 1:dim(tmp)[1]

tmp2 <- tmp
tmp2$y <- tmp$Var2+0.5

tmp3 <- tmp
tmp3$x <- tmp$Var1+0.5
tmp3$y <- tmp$Var2+0.5

tmp4 <- tmp
tmp4$x <- tmp$Var1+0.5

pval.plot <- rbind(tmp, tmp2, tmp3, tmp4)
pval.plot <- pval.plot[order(pval.plot$id),c("id", "x", "y")]

postscript("biomarker_heatmap_cloglog.eps", horizontal = FALSE, onefile = FALSE, paper = "special",
           height = 8, width = 11)

plot2 <- ggplot(dat2, aes(Var1, Var2, fill = value)) + geom_tile() + xlab("biomarker 1") + ylab("biomarker 2")+
  scale_fill_gradient2(low = "blue",  high = "red",limits=c(-1, 1))

plot2 <- plot2+geom_text(aes(fill = dat2$value, label = round(dat2$value, 2)), size=3)


plot2  <- plot2 + theme(plot.margin=unit(c(0,0,0,0),"mm"), axis.ticks = element_blank()) 

plot2 <- plot2+ geom_polygon(aes(x=x, y=y, group=id),fill=NA, data=pval.plot, inherit.aes=FALSE, color = "black")
plot2 <- plot2+
  scale_x_continuous(breaks=cumsum(rep(1, dim(d2)[2])), labels=c("HbA1c", "H2IS", "H2BF", "OGIS", "Glu", "Gln", "Leu", "Ile", "Phe", "Trp", "Tyr", "Val", "C3", "C5M", "C5I", "C5V", "2HA", "Lac", "3HbA", "AKG", "Pyr")) +theme(axis.text.x = element_text(angle = 90, hjust = 1))+
  scale_y_continuous(breaks=cumsum(rep(1, dim(d2)[2])), labels=label(d2))

plot2
dev.off()



######

library(PResiduals)

presid.orm <- function(object){
  k <- object$non.slopes
  L <- object$linear.predictors
  trans <- object$trans
  cumprob <- if(length(trans)) trans$cumprob else plogis
  if(length(L)==0) stop('you did not use linear.predictors=TRUE for the fit')
  if(length(Y <- object$y) == 0) stop("you did not specify y=TRUE in the fit")
  if(!is.factor(Y)) Y <- factor(Y)
  Y <- unclass(Y) - 1
  cof <- object$coefficients
  if(length(X <- object$x)==0)
    stop("you did not use x=TRUE in the fit")
  
  
  N <- length(Y)
  px <- 1 - cumprob(outer(cof[1:k],
                          as.vector(X %*% cof[- (1:k)]), "+"))
  low.x = rbind(0, px)[cbind(Y + 1L, 1:N)]
  hi.x  = 1 - rbind(px, 1)[cbind(Y + 1L, 1:N)]
  return(low.x - hi.x)
}




######  Now fitting models

d$log.art.duration<-with(d, log(art.duration))

dd<-with(d, datadist(age,sex,race,bmi,cd4,log.art.duration,smoker))
options(datadist='dd')

d$y<-d$oa_2_hba 

postscript("quantile-plot.eps",width=10,height=3.5)
par(mfrow=c(1,3))
mod1<-lm(y~age+sex+race+bmi+cd4+log.art.duration+smoker,data=d,subset=hiv==1)
psr1<-presid(mod1)
qqplot(qunif(ppoints(length(psr1)),-1,1),psr1,xlab="Quantiles for Uniform(-1,1)", ylab="PSR", main="Linear Model of Y",cex.lab=1.4,cex.axis=1.4)
abline(0,1)

d$log.y<-log(d$y)
mod2<-lm(log.y~age+sex+race+bmi+cd4+log.art.duration+smoker,data=d,subset=hiv==1)
psr2<-presid(mod2)
qqplot(qunif(ppoints(length(psr2)),-1,1),psr2,xlab="Quantiles for Uniform(-1,1)", ylab="PSR", main="Linear Model of log(Y)",cex.lab=1.4,cex.axis=1.4)
abline(0,1)

mod<-orm(y~age+sex+race+bmi+cd4+log.art.duration+smoker,x=TRUE,y=TRUE,data=d,subset=hiv==1,family="loglog")
psr<-presid(mod)
#psr<-presid.orm(mod)
qqplot(qunif(ppoints(length(psr)),-1,1),psr,xlab="Quantiles for Uniform(-1,1)", ylab="PSR", main="Semiparametric Transformation Model",cex.lab=1.4,cex.axis=1.4)
abline(0,1)
dev.off()


anova(mod)

postscript("biomarker-resid-plot.eps",height=5,width=10)
fval<-.8
par(mfrow=c(2,4),mar=c(4.5,4.2,2,.5))
with(d,plot(age[hiv==1],psr,xlab="Age",ylab="",cex.lab=1.4,cex.axis=1.4))
mtext("PSR",side=2,line=2.2)
with(d,lines(lowess(age[hiv==1],psr,f=fval)))
with(d,plot(bmi[hiv==1],psr,xlab="BMI",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(bmi[hiv==1],psr,f=fval)))
with(d,plot(cd4[hiv==1],psr,xlab="CD4",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(cd4[hiv==1],psr,f=fval)))
with(d,plot(log.art.duration[hiv==1],psr,xlab="ART Duration (log years)",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(log.art.duration[hiv==1],psr,f=fval)))

mod3<-orm(y~rcs(age,3)+sex+race+rcs(bmi,3)+cd4+rcs(log.art.duration,3)+smoker,x=TRUE,y=TRUE,data=d,subset=hiv==1,family="loglog")
psr<-presid.orm(mod3)

anova(mod3)

with(d,plot(age[hiv==1],psr,xlab="Age",ylab="",cex.lab=1.4,cex.axis=1.4))
mtext("PSR",side=2,line=2.2)
with(d,lines(lowess(age[hiv==1],psr,f=fval)))
with(d,plot(bmi[hiv==1],psr,xlab="BMI",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(bmi[hiv==1],psr,f=fval)))
with(d,plot(cd4[hiv==1],psr,xlab="CD4",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(cd4[hiv==1],psr,f=fval)))
with(d,plot(log.art.duration[hiv==1],psr,xlab="ART Duration (log years)",ylab="",cex.lab=1.4,cex.axis=1.4))
with(d,lines(lowess(log.art.duration[hiv==1],psr,f=fval)))
dev.off()

##  AIC for both models
mod3$deviance[2]+2*10
mod$deviance[2]+2*7


