lrm.resid = function(y, z) {
  ## Ensure categories are coded from 1 to ny
  y = as.numeric(as.factor(y))
  if(!is.matrix(z)) z = matrix(z, ncol=1)

  require(rms)

  ny = length(table(y))
  nay = ny - 1
  nz = ncol(z)
  N = length(y)

  ## Fit separate proportional odds models under the null
  modyz = lrm(y ~ z)

  ## Obtain predicted cummulative probabilities.
  ## Each column is for a subject
  py = (1 + exp(outer(modyz$coeff[1:nay],
                      as.vector(z%*%modyz$coeff[-(1:nay)]), "+"))) ^ (-1)

  ## residuals
  low.y = rbind(0,py)[cbind(y,1:N)]
  cum.y = rbind(py,1)[cbind(y,1:N)]
  resid = low.y + cum.y - 1

  ## variances
  ## \sum p_j \gamma_{j-1} \gamma_j =
  ## \sum (\gamma_{j-1}\gamma_j^2) -  \sum (\gamma_{j-1}^2\gamma_j)
  py2 = py^2
  varpy = apply(py*rbind(py2[-1,],1) - py2*rbind(py[-1,],1), 2, sum)

  ## studentized residuals
  student.resid = resid/sqrt(varpy)

  ## latent-variable scale residuals
  G = function(x) log(x/(1-x))
  latentresid = numeric(N)
  for(i in 1:N)
    latentresid[i] = integrate(G, low.y[i], cum.y[i])$value /(cum.y[i]-low.y[i])

  list(modyz = modyz, y = y, z = z,
       residuals = resid, var.resid = varpy, student.resid = student.resid,
       latentresid = latentresid)
}




#######################################################
####  Using Zambia's dataset

##  This dataset doesn't have anything that the dataset that I use doesn't have
#library(foreign)
#d1<-read.dta("Zambia_dataset_cytology.dta")

d<-read.csv("Zambia dataset with condom variables.csv")

age<-d$AgeRAW
educ<-d$EduCat
sexptns<-d$LftmSxPt
condom<-d$Condom4Cat
cd4<-d$CD4RAW
highrhpv<-d$HighRHPV
sexarche<-d$Sexarche
y.dich<-d$HSILCyt
y<-d$Cytology
marstat<-d$MarStat

exclude<-ifelse(is.na(cd4),1,0)
y1<-y[exclude==0]
age1<-age[exclude==0]
cd41<-cd4[exclude==0]
condom1<-condom[exclude==0]
marstat1<-marstat[exclude==0]
educ1<-educ[exclude==0]

zz = lrm.resid(y1, cbind(age1,cd41))

zz1<-lrm.resid(y1, cbind(age1, age1^2, cd41))

sqrt.cd4<-sqrt(cd41)
zz2<-lrm.resid(y1, cbind(age1, age1^2, sqrt.cd4))

table(y)


postscript("residual-by-predictor-plots.eps",height=5,width=9)

par(mfrow=c(1,2),mar=c(5,4,1,1))

with(zz, plot(age1, residuals,xlab="age (years)",ylab="Probability-Scale Residual",ylim=c(-1,1)))
abline(h=0, lty=2); lines(lowess(age1,zz$residuals), col=1, lwd=2)

with(zz1, plot(age1, residuals,xlab="age (years)",ylab="Probability-Scale Residual",ylim=c(-1,1)))
abline(h=0, lty=2); lines(lowess(age1,zz1$residuals), col=1, lwd=2)

dev.off()


library(PResiduals)

mod1<-lrm(y1 ~ age1+cd41, x=TRUE, y=TRUE)
plot(age1,residuals(mod1))
abline(h=0, lty=2); lines(lowess(age1,residuals(mod1)), col=1, lwd=2)
age.sq1<-age1^2
mod2<-lrm(y1 ~ age1+age.sq1+cd41, x=TRUE, y=TRUE)
plot(age1,residuals(mod2))
abline(h=0, lty=2); lines(lowess(age1,residuals(mod2)), col=1, lwd=2)


age.sq<-age^2
cobot(y|condom ~ cd4+age+age.sq+educ+marstat, fisher=TRUE, na.action=na.omit)
cor.test(y1,condom1, method="spearman")



